(function capsule(window) {
  function defineLibrary() {
    const MandelbrotLib = {};
    const Vec3 = VemaLib.Vec3; // eslint-disable-line no-undef
    const Matrix4 = VemaLib.Matrix4; // eslint-disable-line no-undef

    function cubicInterpolate(y0, y1, y2, y3, mu) {
      let a0,
        a1,
        a2,
        a3,
        mu2;

      mu2 = mu * mu;
      a0 = y3 - y2 - y0 + y1;
      a1 = y0 - y1 - a0;
      a2 = y2 - y0;
      a3 = y1;
      return (a0 * mu * mu2 + a1 * mu2 + a2 * mu + a3);
    }

    function cosineInterpolate(y1, y2, mu) {
      const mu2 = (1 - Math.cos(mu * Math.PI)) / 2;
      return y1 * (1 - mu2) + y2 * mu2;
    }

    function linearInterpolate(y1, y2, mu) {
      return y1 * (1 - mu) + y2 * mu;
    }


    class Mandelbrot {
      constructor(gl, canvas) {
        this.params = {};

        this.aspectRatio = 1.0;

        this.roll = 0.0;
        this.zoom = 3.0;
        this.offsetx = -0.5;
        this.offsety = 0.0;
        this.rotate = 0.0;
        this.rotateSpeed = 0.0;
        this.scrollSpeed = 0.015;
        this.colors = [1.0, 0.0, 0.0,
          0.0, 1.0, 0.0,
          0.0, 0.0, 1.0,
          0.0, 1.0, 1.0,
          1.0, 1.0, 0.0,
          0.0, 0.0, 0.0,
          1.0, 1.0, 1.0];

        this.ibuffer = null;
        this.vbuffer = null;
        this.tbuffer = null;
        this.rgb = [0.0, 0.0, 0.0];
      }

      init(gl, c) {
        const vertexShader = `precision highp float;
attribute vec4 vPosition;
attribute highp vec2 multiTexCoord0;
varying highp vec2 texCoord0;
void main() {
  texCoord0 = multiTexCoord0;
  gl_Position = vPosition;
}
`;
        const fragmentShader = `precision highp float;
uniform mat4 uRotMatrix;
varying highp vec2 texCoord0;
uniform highp vec2 centerOffset;
uniform highp float zoom;
uniform highp float colorroll;
uniform highp vec3 colorAdd;
uniform int colorCount;
uniform vec3 colors[9];
const int iterations =255;
const float PI = 3.14159;
highp float mandel(vec2 tc) {
   highp vec2 c = (zoom*(uRotMatrix*vec4(tc ,0.0,0.0)).xy+centerOffset);
   highp vec2 z = c;
   for (int n = 0; n < iterations; n++) {
       z = vec2(z.x * z.x - z.y * z.y, 2.0 * z.x * z.y) + c;
       if (length(z) > 4.0) return (float(n) + 1.0 - log(log(length(z))) / log(2.0));
   }
   return 0.0;
}


void main() {
    highp float n= mandel(texCoord0);
    if (n == 0.0){
      gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
    }else{
       n+=colorroll*50.0;
       vec3 n3 = colorAdd*20.0+n;
       vec3 value = fract(n3*0.02);
       int index0 = int(floor(value.x*float(colorCount)));
       int index1 = int(floor(value.y*float(colorCount)));
       int index2 = int(floor(value.z*float(colorCount)));
       float f0 = value.x*float(colorCount)-float(index0);
       float f1 = value.y*float(colorCount)-float(index1);
       float f2 = value.z*float(colorCount)-float(index2);
       float colorR = 0.0;
       float colorG = 0.0;
       float colorB = 0.0;
       for(int i=0; i<9; i++){
          float c0 = colors[i].r;
          float c1 = (i<colorCount-1)?colors[i+1].r:colors[0].r;
          colorR = (i==index0)?mix(c0,c1,f0):colorR;
          float c2 = colors[i].g;
          float c3 = (i<colorCount-1)?colors[i+1].g:colors[0].g;
          colorG = (i==index1)?mix(c2,c3,f1):colorG;
          float c4 = colors[i].b;
          float c5 = (i<colorCount-1)?colors[i+1].b:colors[0].b;
          colorB = (i==index2)?mix(c4,c5,f2):colorB;
       }
       gl_FragColor.rgb = vec3(colorR,colorG,colorB);
       gl_FragColor.a=1.0;
    }
}`;
        this.program = ShaderLib  // eslint-disable-line no-undef
        .makeProgram(gl, vertexShader, fragmentShader);
        gl.useProgram(this.program);
        this.params.aPosition = gl.getAttribLocation(this.program, 'vPosition');
        this.params.aTexCoordinate = gl.getAttribLocation(this.program, 'multiTexCoord0');
        this.params.uRotMatrix = gl.getUniformLocation(this.program, 'uRotMatrix');
        this.params.colorroll = gl.getUniformLocation(this.program, 'colorroll');
        this.params.zoom = gl.getUniformLocation(this.program, 'zoom');
        this.params.xy = gl.getUniformLocation(this.program, 'centerOffset');
        this.params.colorCount = gl.getUniformLocation(this.program, 'colorCount');
        this.params.colors = gl.getUniformLocation(this.program, 'colors');
        this.params.colorAdd = gl.getUniformLocation(this.program, 'colorAdd');
        const index = 3;
        this.zoom = DataLib.data[index].zoom;
        this.offsetx = DataLib.data[index].offsetx;
        this.offsety = DataLib.data[index].offsety;
        this.rotate = DataLib.data[index].rotate;
        // this.colors = DataLib.data[index].colors;
        this.resize(gl, c);
      }

      makebuffers(gl) {
        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, -1, 1, -1, -1, 1, 1, 1
        ]), gl.STATIC_DRAW);
        this.tbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([
          -this.aspectRatio / 2, -0.5,
          this.aspectRatio / 2, -0.5,
          -this.aspectRatio / 2, 0.5,
          this.aspectRatio / 2, 0.5
        ]), gl.STATIC_DRAW);
        this.ibuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array([0, 1, 2, 1, 2, 3,
        ]), gl.STATIC_DRAW);
      }


      resize(gl, c) {
        this.aspectRatio = c.width / c.height;
        this.makebuffers(gl);
      }


      tick(gl, canvas, time) {
        const timeLength = 100;
        const timeStart = 200;
        const steps = DataLib.fractalPath.length;
        const stepLength = timeLength / steps;


        this.time = time;
        this.t = time - timeStart;
        this.step = Math.floor(this.t / stepLength);
        const mu = (this.t - this.step * stepLength) / stepLength;
        const a = (this.step > 0) ? (this.step - 1) : 0;
        const b = (this.step < steps) ? this.step : steps - 1;
        const c = (b + 1 < steps) ? b + 1 : b;
        const d = (c + 1 < steps) ? c + 1 : c;
        const dataset = DataLib.fractalPath;
        const data0 = dataset[a];
        const data1 = dataset[b];
        const data2 = dataset[c];
        const data3 = dataset[d];

        this.zoom = cosineInterpolate(data1[0], data2[0], mu);
        this.offsetx = cubicInterpolate(data0[1], data1[1], data2[1], data3[1], mu);
        this.offsety = cubicInterpolate(data0[2], data1[2], data2[2], data3[2], mu);
        this.rotate = cubicInterpolate(data0[3], data1[3], data2[3], data3[3], mu);
      }

      setRGB(rgb) {
        this.rgb = rgb;
      }


      draw(gl, canvas, t) {
        gl.useProgram(this.program);
        gl.viewport(0, 0, canvas.width, canvas.height);
        gl.uniform1i(this.params.uTexture, 0); // todo fix this
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(this.params.aPosition);
        gl.vertexAttribPointer(this.params.aPosition, 2, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.enableVertexAttribArray(this.params.aTexCoordinate);
        gl.vertexAttribPointer(this.params.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);
        const r = new Matrix4();
        r.rotationZMatrix(this.rotate);
        gl.uniformMatrix4fv(this.params.uRotMatrix, false, new Float32Array(r.m));
        gl.uniform2f(this.params.xy, this.offsetx, this.offsety);
        gl.uniform1f(this.params.zoom, this.zoom);
        gl.uniform1f(this.params.colorroll, this.roll);
        gl.uniform1i(this.params.colorCount, Math.floor(this.colors.length / 3));
        gl.uniform3fv(this.params.colors, new Float32Array(this.colors));
        gl.uniform3f(this.params.colorAdd, this.rgb[0], this.rgb[1], this.rgb[2]);
        this.roll += this.scrollSpeed;
        if (this.roll > 255.0) this.roll = 0.0;
        if (this.roll < 0.0) this.roll = 255.0;

        this.rotate += this.rotateSpeed;
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
      }
    }

    MandelbrotLib.Mandelbrot = Mandelbrot;

    return MandelbrotLib;
  }
  // define globally if it doesn't already exist
  if (typeof (MandelbrotLib) === 'undefined') {
    const mwindow = window;
    mwindow.MandelbrotLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
